<?php

namespace Drupal\user_referral_webform\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\Plugin\WebformElement\WebformCompositeBase;
use Drupal\user_referral\Entity\UserReferralType;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * Provides a 'referrer' element.
 *
 * @WebformElement(
 *   id = "webform_referrer",
 *   label = @Translation("Referrer"),
 *   description = @Translation("Provides a form element to collect referrer of the visit."),
 *   category = @Translation("Composite elements"),
 *   composite = TRUE,
 *   states_wrapper = TRUE,
 * )
 */
class WebformReferrer extends WebformCompositeBase {

  /**
   * {@inheritdoc}
   */
  protected function formatHtmlItem(array $element, WebformSubmissionInterface $webform_submission, array $options = []) {
    $value = $this->getValue($element, $webform_submission, $options);

    if (empty($value)) {
      return '';
    }

    $format = $this->getItemFormat($element);

    $output = parent::formatHtmlItem($element, $webform_submission, $options);

    $account = User::load($value['referrer_uid']);
    if ($account) {
      $url = Url::fromRoute('entity.user.canonical', ['user' => $account->id()]);
      $output['referrer_user'] = [
        '#type' => 'inline_template',
        '#template' => "<br><b>{{ title }}:</b> {{ value }}",
        '#context' => [
          'title' => t('Referrer User'),
          'value' => Link::fromTextAndUrl($account->label(), $url)->toRenderable(),
        ]
      ];
        
    }

    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function getItemDefaultFormat() {
    return 'link';
  }

  /**
   * {@inheritdoc}
   */
  public function getItemFormats() {
    return parent::getItemFormats() + [
      'link' => $this->t('Link'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setDefaultValue(array &$element) {
    parent::setDefaultValue($element);

    $request = \Drupal::request();

    if ($cookie_data = $request->cookies->get(UserReferralType::COOKIE_NAME)) {
      // Retrieve referral info from the cookie
      $cookie = json_decode($cookie_data);

      if (!empty($cookie)) {
        $element['#default_value'] = [
          'referrer_uid' => $cookie->uid,
          'referral_type' => $cookie->type,
        ];
      }
    }
    
  }

  public function prepare(array &$element, ?WebformSubmissionInterface $webform_submission = NULL) {
    parent::prepare($element, $webform_submission);

    // This internaly computed not accessible to user to fill.
    $element['#access'] = FALSE;
    
  }

  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    // Remove the default section under the advanced tab.
    unset($form['default']);

    unset($form['element_attributes']);
    unset($form['wrapper_attributes']);
    unset($form['label_attributes']);
    
    // This element will not be displayed so question for flexbox.
    // $form['element']['#access'] = FALSE;
    $form['composite']['#access'] = FALSE;
    $form['form']['#access'] = FALSE;
    $form['element_description']['#access'] = FALSE;
    $form['validation']['#access'] = FALSE;
    return $form;
  }

}
